@@ -11,7 +11,7 @@
 *************************************************************************
 ** A TCL Interface to SQLite
 **
-** $Id: tclsqlite.c,v 1.53 2003/12/19 12:32:46 drh Exp $
+** $Id: tclsqlite.c,v 1.59 2004/02/25 22:51:06 rdc Exp $
 */
 #ifndef NO_TCL     /* Omit this whole file if TCL is unavailable */
 
@@ -51,6 +51,7 @@ struct SqliteDb {
   sqlite *db;           /* The "real" database structure */
   Tcl_Interp *interp;   /* The interpreter used for this database */
   char *zBusy;          /* The busy callback routine */
+  char *zCommit;        /* The commit hook callback routine */
   char *zTrace;         /* The trace callback routine */
   char *zProgress;      /* The progress callback routine */
   char *zAuth;          /* The authorization callback routine */
@@ -357,6 +358,23 @@ static void DbTraceHandler(void *cd, const char *zSql){
   Tcl_ResetResult(pDb->interp);
 }
 
+/*
+** This routine is called when a transaction is committed.  The
+** TCL script in pDb->zCommit is executed.  If it returns non-zero or
+** if it throws an exception, the transaction is rolled back instead
+** of being committed.
+*/
+static int DbCommitHandler(void *cd){
+  SqliteDb *pDb = (SqliteDb*)cd;
+  int rc;
+
+  rc = Tcl_Eval(pDb->interp, pDb->zCommit);
+  if( rc!=TCL_OK || atoi(Tcl_GetStringResult(pDb->interp)) ){
+    return 1;
+  }
+  return 0;
+}
+
 /*
 ** This routine is called to evaluate an SQL function implemented
 ** using TCL script.
@@ -468,19 +486,23 @@ static int auth_callback(
 static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
   SqliteDb *pDb = (SqliteDb*)cd;
   int choice;
+  int rc = TCL_OK;
   static const char *DB_strs[] = {
-    "authorizer",         "busy",              "changes",
-    "close",              "complete",          "errorcode",
-    "eval",               "function",          "last_insert_rowid",
-    "onecolumn",          "timeout",            "trace",
-    "progress",           0
+    "authorizer",         "busy",                   "changes",
+    "close",              "commit_hook",            "complete",
+    "errorcode",          "eval",                   "function",
+    "last_insert_rowid",  "last_statement_changes", "onecolumn",
+    "progress",           "rekey",                  "timeout",
+    "trace",
+    0                    
   };
   enum DB_enum {
-    DB_AUTHORIZER,        DB_BUSY,             DB_CHANGES,
-    DB_CLOSE,             DB_COMPLETE,         DB_ERRORCODE,
-    DB_EVAL,              DB_FUNCTION,         DB_LAST_INSERT_ROWID,
-    DB_ONECOLUMN,         DB_TIMEOUT,          DB_TRACE,            
-    DB_PROGRESS
+    DB_AUTHORIZER,        DB_BUSY,                   DB_CHANGES,
+    DB_CLOSE,             DB_COMMIT_HOOK,            DB_COMPLETE,
+    DB_ERRORCODE,         DB_EVAL,                   DB_FUNCTION,
+    DB_LAST_INSERT_ROWID, DB_LAST_STATEMENT_CHANGES, DB_ONECOLUMN,        
+    DB_PROGRESS,          DB_REKEY,                  DB_TIMEOUT,
+    DB_TRACE
   };
 
   if( objc<2 ){
@@ -640,6 +662,26 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
     break;
   }
 
+  /*
+  **     $db last_statement_changes
+  **
+  ** Return the number of rows that were modified, inserted, or deleted by
+  ** the last statment to complete execution (excluding changes due to
+  ** triggers)
+  */
+  case DB_LAST_STATEMENT_CHANGES: {
+    Tcl_Obj *pResult;
+    int lsChange;
+    if( objc!=2 ){
+      Tcl_WrongNumArgs(interp, 2, objv, "");
+      return TCL_ERROR;
+    }
+    lsChange = sqlite_last_statement_changes(pDb->db);
+    pResult = Tcl_GetObjResult(interp);
+    Tcl_SetIntObj(pResult, lsChange);
+    break;
+  }
+
   /*    $db close
   **
   ** Shutdown the database
@@ -649,6 +691,43 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
     break;
   }
 
+  /*    $db commit_hook ?CALLBACK?
+  **
+  ** Invoke the given callback just before committing every SQL transaction.
+  ** If the callback throws an exception or returns non-zero, then the
+  ** transaction is aborted.  If CALLBACK is an empty string, the callback
+  ** is disabled.
+  */
+  case DB_COMMIT_HOOK: {
+    if( objc>3 ){
+      Tcl_WrongNumArgs(interp, 2, objv, "?CALLBACK?");
+    }else if( objc==2 ){
+      if( pDb->zCommit ){
+        Tcl_AppendResult(interp, pDb->zCommit, 0);
+      }
+    }else{
+      char *zCommit;
+      int len;
+      if( pDb->zCommit ){
+        Tcl_Free(pDb->zCommit);
+      }
+      zCommit = Tcl_GetStringFromObj(objv[2], &len);
+      if( zCommit && len>0 ){
+        pDb->zCommit = Tcl_Alloc( len + 1 );
+        strcpy(pDb->zCommit, zCommit);
+      }else{
+        pDb->zCommit = 0;
+      }
+      if( pDb->zCommit ){
+        pDb->interp = interp;
+        sqlite_commit_hook(pDb->db, DbCommitHandler, pDb);
+      }else{
+        sqlite_commit_hook(pDb->db, 0, 0);
+      }
+    }
+    break;
+  }
+
   /*    $db complete SQL
   **
   ** Return TRUE if SQL is a complete SQL statement.  Return FALSE if
@@ -692,7 +771,6 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
     CallbackData cbData;
     char *zErrMsg;
     char *zSql;
-    int rc;
 #ifdef UTF_TRANSLATION_NEEDED
     Tcl_DString dSql;
     int i;
@@ -810,7 +888,6 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
   ** Return a single column from a single row of the given SQL query.
   */
   case DB_ONECOLUMN: {
-    int rc;
     char *zSql;
     char *zErrMsg = 0;
     if( objc!=3 ){
@@ -820,7 +897,7 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
     zSql = Tcl_GetStringFromObj(objv[2], 0);
     rc = sqlite_exec(pDb->db, zSql, DbEvalCallback3, interp, &zErrMsg);
     if( rc==SQLITE_ABORT ){
-      /* Do nothing.  This is normal. */
+      rc = SQLITE_OK;
     }else if( zErrMsg ){
       Tcl_SetResult(interp, zErrMsg, TCL_VOLATILE);
       free(zErrMsg);
@@ -832,6 +909,29 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
     break;
   }
 
+  /*
+  **     $db rekey KEY
+  **
+  ** Change the encryption key on the currently open database.
+  */
+  case DB_REKEY: {
+    int nKey;
+    void *pKey;
+    if( objc!=3 ){
+      Tcl_WrongNumArgs(interp, 2, objv, "KEY");
+      return TCL_ERROR;
+    }
+    pKey = Tcl_GetByteArrayFromObj(objv[2], &nKey);
+#ifdef SQLITE_HAS_CODEC
+    rc = sqlite_rekey(pDb->db, pKey, nKey);
+    if( rc ){
+      Tcl_AppendResult(interp, sqlite_error_string(rc), 0);
+      rc = TCL_ERROR;
+    }
+#endif
+    break;
+  }
+
   /*
   **     $db timeout MILLESECONDS
   **
@@ -885,11 +985,11 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
   }
 
   } /* End of the SWITCH statement */
-  return TCL_OK;
+  return rc;
 }
 
 /*
-**   sqlite DBNAME FILENAME ?MODE?
+**   sqlite DBNAME FILENAME ?MODE? ?-key KEY?
 **
 ** This is the main Tcl command.  When the "sqlite" Tcl command is
 ** invoked, this routine runs to process that command.
@@ -919,21 +1019,34 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
 **       not.  Used by tests to make sure the library was compiled 
 **       correctly.
 */
-static int DbMain(void *cd, Tcl_Interp *interp, int argc, char **argv){
+static int DbMain(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
   int mode;
   SqliteDb *p;
+  void *pKey = 0;
+  int nKey = 0;
+  const char *zArg;
   char *zErrMsg;
+  const char *zFile;
   char zBuf[80];
-  if( argc==2 ){
-    if( strcmp(argv[1],"-encoding")==0 ){
+  if( objc==2 ){
+    zArg = Tcl_GetStringFromObj(objv[1], 0);
+    if( strcmp(zArg,"-encoding")==0 ){
       Tcl_AppendResult(interp,sqlite_encoding,0);
       return TCL_OK;
     }
-    if( strcmp(argv[1],"-version")==0 ){
+    if( strcmp(zArg,"-version")==0 ){
       Tcl_AppendResult(interp,sqlite_version,0);
       return TCL_OK;
     }
-    if( strcmp(argv[1],"-tcl-uses-utf")==0 ){
+    if( strcmp(zArg,"-has-codec")==0 ){
+#ifdef SQLITE_HAS_CODEC
+      Tcl_AppendResult(interp,"1",0);
+#else
+      Tcl_AppendResult(interp,"0",0);
+#endif
+      return TCL_OK;
+    }
+    if( strcmp(zArg,"-tcl-uses-utf")==0 ){
 #ifdef TCL_UTF_MAX
       Tcl_AppendResult(interp,"1",0);
 #else
@@ -942,14 +1055,26 @@ static int DbMain(void *cd, Tcl_Interp *interp, int argc, char **argv){
       return TCL_OK;
     }
   }
-  if( argc!=3 && argc!=4 ){
-    Tcl_AppendResult(interp,"wrong # args: should be \"", argv[0],
-       " HANDLE FILENAME ?MODE?\"", 0);
+  if( objc==5 || objc==6 ){
+    zArg = Tcl_GetStringFromObj(objv[objc-2], 0);
+    if( strcmp(zArg,"-key")==0 ){
+      pKey = Tcl_GetByteArrayFromObj(objv[objc-1], &nKey);
+      objc -= 2;
+    }
+  }
+  if( objc!=3 && objc!=4 ){
+    Tcl_WrongNumArgs(interp, 1, objv, 
+#ifdef SQLITE_HAS_CODEC
+      "HANDLE FILENAME ?-key CODEC-KEY?"
+#else
+      "HANDLE FILENAME ?MODE?"
+#endif
+    );
     return TCL_ERROR;
   }
-  if( argc==3 ){
+  if( objc==3 ){
     mode = 0666;
-  }else if( Tcl_GetInt(interp, argv[3], &mode)!=TCL_OK ){
+  }else if( Tcl_GetIntFromObj(interp, objv[3], &mode)!=TCL_OK ){
     return TCL_ERROR;
   }
   zErrMsg = 0;
@@ -959,14 +1084,20 @@ static int DbMain(void *cd, Tcl_Interp *interp, int argc, char **argv){
     return TCL_ERROR;
   }
   memset(p, 0, sizeof(*p));
-  p->db = sqlite_open(argv[2], mode, &zErrMsg);
+  zFile = Tcl_GetStringFromObj(objv[2], 0);
+#ifdef SQLITE_HAS_CODEC
+  p->db = sqlite_open_encrypted(zFile, pKey, nKey, 0, &zErrMsg);
+#else
+  p->db = sqlite_open(zFile, mode, &zErrMsg);
+#endif
   if( p->db==0 ){
     Tcl_SetResult(interp, zErrMsg, TCL_VOLATILE);
     Tcl_Free((char*)p);
     free(zErrMsg);
     return TCL_ERROR;
   }
-  Tcl_CreateObjCommand(interp, argv[1], DbObjCmd, (char*)p, DbDeleteCmd);
+  zArg = Tcl_GetStringFromObj(objv[1], 0);
+  Tcl_CreateObjCommand(interp, zArg, DbObjCmd, (char*)p, DbDeleteCmd);
 
   /* The return value is the value of the sqlite* pointer
   */
@@ -1008,13 +1139,13 @@ static int DbMain(void *cd, Tcl_Interp *interp, int argc, char **argv){
 */
 int Sqlite_Init(Tcl_Interp *interp){
   Tcl_InitStubs(interp, "8.0", 0);
-  Tcl_CreateCommand(interp, "sqlite", (Tcl_CmdProc*)DbMain, 0, 0);
+  Tcl_CreateObjCommand(interp, "sqlite", (Tcl_ObjCmdProc*)DbMain, 0, 0);
   Tcl_PkgProvide(interp, "sqlite", "2.0");
   return TCL_OK;
 }
 int Tclsqlite_Init(Tcl_Interp *interp){
   Tcl_InitStubs(interp, "8.0", 0);
-  Tcl_CreateCommand(interp, "sqlite", (Tcl_CmdProc*)DbMain, 0, 0);
+  Tcl_CreateObjCommand(interp, "sqlite", (Tcl_ObjCmdProc*)DbMain, 0, 0);
   Tcl_PkgProvide(interp, "sqlite", "2.0");
   return TCL_OK;
 }
